!
! vertical interpolation routines
!  b = log(a)

	pure subroutine log_array(a,b,ix,nlev)

	integer, intent(in) :: ix, nlev
	real, intent(in) :: a(ix,nlev)
	real, intent(out) ::  b(ix,nlev)
	integer :: i, j

!OMP PARALLEL DO PRIVATE (i,j)
	do j = 1, nlev
	    do i = 1, ix
		b(i,j) = log(a(i,j))
	    enddo
	enddo

	return
	end

!
! find levels
!
	pure subroutine find_level(x,lev,p,ix,nlev)

	integer, intent(in) :: ix, nlev
	real, intent(in) :: x, p(ix,nlev)
	real, intent(out) :: lev(ix)
	integer :: i, j, low,mid,high

!   should parallize calling routine
!    !OMP PARALLEL DO PRIVATE (i,low,mid,high)
	do i = 1, ix
	    if (x > p(i,1)) then
		lev(i) = 0.0
		cycle
	    endif
	    if (x < p(i,nlev)) then
		lev(i) = nlev + 1.0
		cycle
	    endif
	    low = 1
	    high = nlev
	    do while (high-low > 1)
		mid = (high + low) / 2
		if (x <= p(i,mid)) then
		    low = mid
		else
		    high = mid
		endif
	    enddo
	    lev(i) = low + (x-p(i,low))/(p(i,high) - p(i,low))
	enddo
	return
	end

!
!  vertical interpolation: levels -> pressure-level values
!
	pure subroutine get_pvalues(x,lev,p,ix,nlev)

	integer, intent(in) :: ix, nlev
	real, intent(out) :: x(ix)
	real, intent(in) :: lev(ix), p(ix,nlev)
	real :: frac
	integer :: ilev
!OMP PARALLEL DO PRIVATE (i,frac,ilev)
	do i = 1, ix
	    if (lev(i).eq.0 .or. lev(i) > nlev) then
		x(i) = 9.999e20
	    else
		ilev = int(lev(i))
		frac = lev(i) - ilev
		if (ilev .eq. nlev) then
		    x(i) = p(i,nlev)
		else
		    x(i) = (1.0-frac)*p(i,ilev) + frac*p(i,ilev+1)
		endif
	    endif
	enddo
	return
	end

